package eu.mathiasl.legacychatfix;

import com.comphenix.protocol.PacketType;
import com.comphenix.protocol.events.ListenerPriority;
import com.comphenix.protocol.events.PacketAdapter;
import com.comphenix.protocol.events.PacketEvent;
import com.comphenix.protocol.reflect.StructureModifier;
import com.comphenix.protocol.wrappers.WrappedChatComponent;
import net.md_5.bungee.api.ChatColor;
import net.md_5.bungee.api.chat.BaseComponent;
import net.md_5.bungee.api.chat.TextComponent;
import net.md_5.bungee.chat.ComponentSerializer;
import org.bukkit.plugin.Plugin;

import java.util.HashMap;

public class LegacyChatFixPacketAdapter extends PacketAdapter {

    /**
     * Legacy color codes to replace.
     * TODO: Add support for custom legacy color codes through
     *  the config file.
     */
    private final HashMap<String, String> legacyColors = new HashMap<String, String>() {
        {
            put("\\<black>", "black");
            put("\\<dark_blue>", "dark_blue");
            put("\\<dark_green>", "dark_green");
            put("\\<dark_aqua>", "dark_aqua");
            put("\\<dark_red>", "dark_red");
            put("\\<dark_purple>", "dark_purple");
            put("\\<gold>", "gold");
            put("\\<gray>", "gray");
            put("\\<dark_gray>", "dark_gray");
            put("\\<blue>", "blue");
            put("\\<green>", "green");
            put("\\<aqua>", "aqua");
            put("\\<red>", "red");
            put("\\<light_purple>", "light_purple");
            put("\\<yellow>", "yellow");
            put("\\<white>", "white");
            put("\\<bold>", "bold");
            put("\\<strikethrough>", "strikethrough");
            put("\\<underline>", "underline");
            put("\\<italic>", "italic");
            put("\\<magic>", "obfuscated");
            put("\\<secondary>", "gray");
        }
    };

    /**
     * Set the adapter to listen for packets being sent to the client from "system chat".
     * @param plugin A reference to the plugin class responsible for the adapter.
     */
    public LegacyChatFixPacketAdapter(Plugin plugin) {
        super(plugin, ListenerPriority.NORMAL, PacketType.Play.Server.SYSTEM_CHAT);
    }

    /**
     * Listen for packets being sent to the client and replace legacy color codes.
     *
     * @param event  The packet event being sent.
     */
    @Override
    public void onPacketSending(PacketEvent event) {
        // Get the chat components from the packet
        StructureModifier<WrappedChatComponent> chatComponents = event.getPacket().getChatComponents();
        // If there are no chat components, return
        if (chatComponents == null || chatComponents.size() == 0) return;

        // Get the chat component
        WrappedChatComponent chatComponent = chatComponents.read(0); // Get the chat component
        // Convert the chat component to a JSON string
        String message = chatComponent.getJson(); // Get the message
        // Parse the message (to check if it's a chat component)
        BaseComponent[] bungeeComponents;
        try {
            bungeeComponents = ComponentSerializer.parse(message); // Parse the message (to check if it's a chat component
        } catch (Exception e) {
            plugin.getLogger().warning("Failed to parse chat component: " + message);
            return;
        }

        // If no bungee components are found after all, return
        if (bungeeComponents.length == 0) return;

        // Iterate through the bungee components and replace legacy color codes
        for (BaseComponent component : bungeeComponents) {
            // If the component is not a text component, continue
            if (!(component instanceof TextComponent))
                continue;

            // Replace legacy color codes in the text component
            TextComponent textComponent = (TextComponent) component;
            replaceLegacyColor(textComponent, legacyColors);

            // If the text component has extra components, replace legacy color codes in those as well
            for (BaseComponent extra : textComponent.getExtra()) {
                if (!(extra instanceof TextComponent))
                    continue;

                TextComponent extraTextComponent = (TextComponent) extra;
                replaceLegacyColor(extraTextComponent, legacyColors);
            }
        }
        // Write the modified chat components back to the packet
        chatComponents.write(0, WrappedChatComponent.fromJson(ComponentSerializer.toString(bungeeComponents)));
    }

    /**
     * Replaces legacy color code formatting with modern color formatting in
     * through the TextComponent and its extra components.
     *
     * @param textComponent  The text component to replace the legacy color codes in.
     * @param legacyColors   The legacy color codes to replace.
     */
    private void replaceLegacyColor(TextComponent textComponent, HashMap<String, String> legacyColors) {
        // Get the text from the text component
        String text = textComponent.getText();

        // Iterate through the legacy color codes and replace them with modern color codes
        for (String legacyColor : legacyColors.keySet()) {
            // If the text does not contain the legacy color code, continue
            if (!text.contains(legacyColor))
                continue;
            // Replace the legacy color code with an empty string
            text = text.replace(legacyColor, "");
            // Set the text component's text to the modified text
            textComponent.setText(text);
            // Set the text component's color to "RESET" initially
            ChatColor chatColor = ChatColor.RESET;
            try {
                // Try to set the text component's color to the modern color code
                chatColor = ChatColor.of(legacyColors.get(legacyColor));
            } catch (IllegalArgumentException e) {
                // If the modern color code is invalid, log a warning
                this.getPlugin().getLogger().warning("Invalid color code: " + legacyColors.get(legacyColor));
            }
            // Set the text component's color to the modern color code
            textComponent.setColor(chatColor);
        }
    }
}
